--  file  : font.ex - demo/tutorial
--  author: jiri babor
--  email : jbabor@paradise.net.nz
--  date  : 98-11-05

--  date  : 98-12-04
--  font version 4.40: reflects expanded set_parameter routine

include font.e
include get.e
include image.e

constant
    TRUE=1,
    ENTER=13,
    ESC=27,
    UP=328,
    DOWN=336,
    KEYS={#10A&"p"&#10C&"revious   ",#10A&"n"&#10C&"ext   ",
          #10A&"c"&#10C&"ontents   ",#10A&"q"&#10C&"uit   "}
object junk, tile
sequence allblack,circles,circles_pal,pal
atom t
integer mo28,page,sc24,sysb

procedure bar(integer x,integer y,integer w,integer h,object c)
    -- filled rectangle of color c, defined by top left corner
    -- coordinates {x,y}, width w, and height h
    -- c is either integer, indicating fill color, or 2-dimensional
    -- texture color sequence
    sequence s
    integer lx,ly,mx,my,n
    if sequence(c) then -- textured bar
        ly=length(c)
        lx=length(c[1])
        n=floor(w/lx)
        s=c
        for i=1 to ly do
            for j=1 to n do
                s[i]=s[i]&c[i]
            end for
        end for
        mx=1+remainder(x,lx)
        for i=1 to ly do
            s[i]=c[i][mx..lx] & s[i]
        end for
        for r=y to y+h-1 do
            my=1+remainder(r,ly)
            pixel(s[my][1..w],{x,r})
        end for
    else polygon(c,1,{{x,y},{x+w-1,y},{x+w-1,y+h-1},{x,y+h-1}})
    end if
end procedure -- bar

procedure cls(object c)
    sequence vc
    integer h,w
    vc=video_config()
    w=vc[VC_XPIXELS]
    h=vc[VC_YPIXELS]
    bar(0,0,w,h,c)
end procedure

procedure header(sequence s)
    cls(BLUE)
    select_font(mo28)
    set_attributes(SHADOW)
    set_foreground(YELLOW)
    set_background(CLEAR)
    set_shadow_color(BLACK)
    setx(10) sety(15)
    write(s)
end procedure

procedure footer(sequence keys,integer pg)
    sequence s
    integer key
    s={}
    for i=1 to length(keys) do
        key=find(keys[i],"pncq")
        if key then
            s=s & KEYS[key]
        end if
    end for
    keys=keys & ESC     -- add Esc key
    select_font(sysb)
    set_attributes(NONE)
    set_background(BLUE)
    setx(10) sety(460)
    write(s)
    setx(580)
    write("page " & sprintf("%d",pg))
    key=wait_key()
    while not find(key,keys) do
        key=wait_key()
    end while
    if key='n' then page=pg+1
    elsif key='p' then page=pg-1
    elsif key='c' then page=1
    elsif key='q' or key=ESC then
        junk=graphics_mode(-1)
        abort(1)
    end if
end procedure

procedure set_fire_palette()
    pal=get_all_palette()           -- default palette
    -- yellow-red-slate fire palette
    for i = 1 to 16 do
        pal[i+128]=floor({i-1,(i-1)/2,(i-1)/2})
        pal[i+144]=floor({15+i,8-i/2,8-i/2})
    end for
    for i = 1 to 32 do
        pal[i+160]=floor({31+i,i-1,0})
        pal[i+192]=floor({63,31+i,i-1})
        pal[i+224]=floor({63,63,31+i})
    end for
end procedure

procedure fire(integer x,integer y,integer w,integer h)
    sequence buff,cs,s
    atom t
    integer a,A,n,fc

    h=h+30                      -- total height of flame box
    y=y-30                      -- top of flame box...
    buff=repeat(repeat(0,w),h)  -- buffer corresponding to flame box
    cs=repeat(30,w)             -- fire seed vertical position in flame box
    A=#A0000+320*y+x            -- address of top left corner of flame box

    -- read text pixels into buffer
    a=A+31*320
    for i=31 to h do
        a=a+320
        buff[i]=peek({a,w})
    end for
    n=0                         -- flame columns count, reached last row
    while n<w-2 do
        t=time()+0.04           -- 25 fps max
        -- seed flames
        for i=2 to w-1 do
            if rand(3)=1 and cs[i]<h then      -- advance flame
                cs[i]=cs[i]+1
                if cs[i]=h then
                    n=n+1
                end if
            end if
            if cs[i]<h then
                if buff[cs[i]][i] then
                    buff[cs[i]-1][i]=127+rand(128)
                    buff[cs[i]][i]=191+rand(64)
                end if
            end if
        end for

        -- flame calcs
        for r=2 to h-1 do
            for c=2 to w-1 do
                if r<cs[c] then
                    s=buff[r]
                    fc=floor(0.25*(s[c-1]+s[c]+s[c+1]+buff[r+1][c]))
                    if fc>128 then fc=fc-1      -- attenuate
                    else fc=128
                    end if
                    buff[r-1][c]=fc
                end if
            end for
        end for

        -- show buffer
        a=A+640
        for r=2 to h do
            a=a+320
            poke(a,buff[r])
        end for
        while time()<t do end while -- waste time
    end while
end procedure

procedure burn()
    junk=graphics_mode(19)
    cls(BLACK)
    set_fire_palette()
    -- write in black on black, invisibly
    all_palette(allblack)
    select_font(mo28)
    set_foreground(tile)
    set_background(CLEAR)
    setx(160) sety(85)
    justify(CENTRE)
    write("font demo")
    justify(LEFT)                   -- restore default justification
    -- fade-in over 50 steps
    for i=1 to 50 do
        t=time()+0.06               -- 3 second fade-in
        all_palette(floor(0.02*i*pal))
        while time()<t do end while -- waste time
    end for
    fire(102,85,114,23)
    page=1
    junk=graphics_mode(18)
end procedure

procedure contents()
    sequence cs
    integer item,key,len

    header("demo/tutorial : contents")

    cs={{"burn",             0},
        {"contents",         1},
        {"introduction",     2},
        {"basics",           3},
        {"modes",            6},
        {"attributes",       7},
        {"justification",    9},
        {"orientation",     10},
        {"parameters",      12},
        {"prompt",          13},
        {"odds and ends",   14},
        {"the last word",   15}}

    item=3

    select_font(sysb)
    set_background(BLUE)
    len=length(cs)
    for i=1 to len do
        if i=item then
            set_foreground(GREEN)
        else
            set_foreground(BRIGHT_WHITE)
        end if
        setx(100) sety(100+22*i)
        write(cs[i][1])
        setx(240)
        write(sprintf("%2d",cs[i][2]))
    end for

    -- footer
    set_foreground(BRIGHT_RED)
    setx(10) sety(460)
    write(#10A&"p"&#10C&"revious  "&#10A&"n"&#10C&"ext  or use  "&#10A&
          "enter"&#10C&"  and  "&#10A&"up"&#10C&"  and  "&#10A&
          "down"&#10C&"  arrow keys to select,  "&#10A&"esc"&#10C&
          "  to  "&#10A&"q"&#10C&"uit")
    setx(580)
    write("page 1")

    while TRUE do
        key=wait_key()
        if key=UP then
            set_foreground(BRIGHT_WHITE)
            setx(100) sety(100+22*item)
            write(cs[item][1])
            setx(240)
            write(sprintf("%2d",cs[item][2]))
            item=item-1
            if item<1 then item=len
            elsif item=2 then item=1
            end if
            set_foreground(GREEN)
            setx(100) sety(100+22*item)
            write(cs[item][1])
            setx(240)
            write(sprintf("%2d",cs[item][2]))
        elsif key=DOWN then
            set_foreground(BRIGHT_WHITE)
            setx(100) sety(100+22*item)
            write(cs[item][1])
            setx(240)
            write(sprintf("%2d",cs[item][2]))
            item=item+1
            if item>len then item=1
            elsif item=2 then item=3
            end if
            set_foreground(GREEN)
            setx(100) sety(100+22*item)
            write(cs[item][1])
            setx(240)
            write(sprintf("%2d",cs[item][2]))
        elsif key='p' then  -- previous:burn
            page=0
            exit
        elsif key='n' then  -- next:introduction
            page=2
            exit
        elsif key=ENTER then
            page=cs[item][2]
            exit
        elsif key=ESC or key='q' then
            junk=graphics_mode(-1)
            abort(1)
        end if
    end while
    -- footer
end procedure

procedure page2()
    header("demo/tutorial : introduction")
    select_font(sysb)
    set_foreground(BRIGHT_WHITE)
    set_background(BLUE)
    setx(10) sety(60)
    write("Welcome!\n\n")
    write("This demo is meant as a quick introduction to my font module for the benefit of the great majority\n")
    write("of Euphorians that stubbornly refuse to read the docs!\n\n")
    write("My comments will appear in  "&-1&"brigh white"&0&"  in the current font.\n")
    write("Code examples will be usually indented, in  "&-1&#10E&"yellow"&0&#10F&",  using the default ROM font, like this:\n\n")
    select_font(ROM)
    set_foreground(10)
    write("    write(\"Welcome...\")\n\n")
    select_font(sysb)
    set_foreground(15)
    write("Everything else could be just anything, special effects, a lot of bull...\n\n")
    write("I shall start with the shortest program example I can think of:\n\n")
    select_font(ROM)
    set_foreground(14)
    write("    include font.e\n    object junk\n    junk=graphics_mode(18)\n" &
          "    write(\"Hello, world!\")\n\n")
    select_font(sysb)
    set_foreground(BRIGHT_WHITE)
    write("And this is what you will see in the top left corner of your screen:\n\n")
    bar(10,395,620,50,BLACK)
    select_font(ROM)
    set_background(BLACK)
    setx(10) sety(395)
    write("Hello, world!")

    footer("pncq",2)
end procedure

procedure page3()
    header("demo/tutorial : basics")
    select_font(sysb)
    set_foreground(BRIGHT_WHITE)
    set_background(BLUE)
    setx(10) sety(60)
    write("The example on the previous page is quite obnoxious, for several reasons, just one of them being\n")
    write("it does not have a graceful exit!  - Sorry...\n\n")

    write("Just a few comments: The font module works only in the graphic modes that support Euphoria's pixel\n")
    write("routine, not in any of the text modes! It includes a set of sensible defaults, some of them came into\n")
    write("play already in the previous example: "&-1&"8x16 ROM"&0&"  font is "&-1&"automatically"&0&" loaded and immediately\n")
    write("available. The text pointer, a pair of x-y coordinates indicating the top left hand corner of the next\n")
    write("char space is initially set to {0,0}, the foreground is set to BRIGHT_WHITE, the background to CLEAR.\n\n")

    write("Literally hundreds of fonts of all sorts of shapes and sizes are already available in the suitable\n")
    write("format, many of them are also included in this package. The chosen font has to be loaded first:\n\n")
    select_font(ROM)
    set_foreground(YELLOW)
    write("    integer mo28\n    mo28 = load_font(\"modern28.f\")\n\n")
    select_font(sysb)
    set_foreground(BRIGHT_WHITE)
    write("The function loads the font into the memory (in this case from the current directory) and returns a\n")
    write("font handle, which can be used at any stage to select the font as "&-1&"current"&-0&", as in the next example:\n\n")
    select_font(ROM)
    set_foreground(YELLOW)
    write("    select_font(mo28)\n    set_attributes(SHADOW)\n    set_foreground(YELLOW)\n    set_background(CLEAR)\n")
    write("    setx(10) sety(15)\n    write(\"demo/tutorial : basics\")\n\n")

    footer("pncq",3)
end procedure

procedure page4()
    header("demo/tutorial : basics")
    select_font(sysb)
    set_foreground(BRIGHT_WHITE)
    set_background(BLUE)
    setx(10) sety(60)
    write("The example at the bottom of the previous screen happens to be an almost complete subroutine for\n")
    write("the page header!\n\n")
    write("The first line re-selects my favorite font, in the second line the SHADOW attribute is re-activated.\n")
    write("The third sets the yellow foreground (text color) and the fourth invokes, implicitly through the\n")
    write("selection of the CLEAR background, the slower "&-1&"transparent"&0&" mode (we will talk more about various\n")
    write("modes soon in a dedicated section!)\n\n")
    write("setx() and sety() routines move the text pointer to the desired starting location. The text pointer is\n")
    write(-1&"automatically"&0&" updated by every write operation, and its current value can also be retrieved using\n")
    write("corresponding getx() and gety() functions.\n\n")
    write("The write() procedure is the main routine of the system. Its behavior depends on current attributes,\n")
    write("font parameters and the line orientation as well as justification.\n\n")

    write("The text foreground can be either a solid single color, or any two-dimensional color pattern, a tile.\n\n")
    write("The foreground color can also be changed 'on-the-fly', from within the text itself. Just a simple\n")
    write("example:\n\n")
    select_font(ROM)
    set_foreground(YELLOW)
    write("    select(sc24)\n")
    write("    set_foreground(GREEN)\n")
    write("    set_background(CLEAR)\n")
    write("    write(\"New font: \"&#10C&\" scotia24.f\\n\\n\")\n")

    footer("pncq",4)
end procedure

procedure page5()
    header("demo/tutorial : basics")
    select_font(sysb)
    set_foreground(BRIGHT_WHITE)
    set_background(BLUE)
    setx(10) sety(60)
    write("And this is what you get:\n\n")
    select_font(sc24)
    set_foreground(GREEN)
    set_background(CLEAR)
    write("New font: "&#10C&" scotia24.f\n\n")
    select_font(sysb)
    set_foreground(BRIGHT_WHITE)
    set_background(BLUE)
    write("The first part is in GREEN, as stipulated by the set_foreground() statement, then it changes to\n")
    write("BRIGHT_RED. The culprit is the imbedded character #10C in hex, or 268 in decimal. Here's how it works:\n")
    write("any character greater than 255 resets the foreground color according to this formula:\n\n")
    write(-1&"    new_color = char - 256\n\n"&0)
    write("So in our case  268 - 256 = 12 , the code for BRIGHT_RED! Obviously, only plain colors can be imbedded,\n")
    write("not tiles!  And since I mentioned tiles again, do you remember the colorful set of the opening screen?\n")
    write("That was tiles! Let's try another, simpler example:\n\n")
    select_font(ROM)
    set_foreground(YELLOW)
    write("    select(sc24)\n")
    write("    tile = {{4,4,2,2},{2,4,4,2},{2,2,4,4},{4,2,2,4}}\n")
    write("    set_foreground(tile)\n")
    write("    write(\"Ugly diagonal red and green stripes...\")\n\n")
    select_font(sysb)
    set_foreground(BRIGHT_WHITE)
    write("And this is how it looks:\n\n")
    select_font(sc24)
    tile = {{4,4,2,2},{2,4,4,2},{2,2,4,4},{4,2,2,4}}
    set_foreground(tile)
    set_background(CLEAR)
    write("Ugly diagonal red and green stripes...\n\n")

    footer("pncq",5)
end procedure

procedure page6()
    header("demo/tutorial : modes")
    select_font(sysb)
    set_foreground(BRIGHT_WHITE)
    set_background(BLUE)
    setx(10) sety(60)
    write("If you look again at the code for the last example, you will notice we did not change the background.\n\n")
    write("And it is the "&-1&"background color"&0&" that dictates the two main modes: the slower, but more versatile\n")
    write(-1&"transparent"&0&" mode, which is implicitly invoked every time we choose "&-1&"CLEAR"&0&" background, and the much\n")
    write("faster "&-1&"opaque"&0&" mode, automatically set when a "&-1&"plain color"&0&" background is chosen.\n\n")
    write("Just a little demo to illustrate the difference between "&-1&"transparent"&0&" and "&-1&"opaque"&0&" backgrounds:")
    bar(10,195,620,120,circles)
    select_font(mo28)
    set_attributes(NONE)
    set_foreground(YELLOW)
    set_background(CLEAR)
    setx(30) sety(220)
    write("This text is written with "&#10F&"transparent"&#10E&" background...")
    set_background(BLACK)
    setx(30) sety(260)
    write("And this text is written with "&#10F&"opaque"&#10E&" background...")

    select_font(sysb)
    set_foreground(BRIGHT_WHITE)
    set_background(BLUE)
    setx(10) sety(340)
    write("Can you see the difference?  - The second line has a "&-1&"black"&0&" background obliterating the blue circles.\n")
    write("That's what I meant when I said the opaque mode's less versatile...\n\n")
    write("But the opaque mode is much faster in some graphics modes - like this one, mode 18 - mainly because\n")
    write("the whole line is written to the screen at once, as a "&-1&"single image"&0&", while in the clear mode, the text \n")
    write("is built character by character, virtually pixel by pixel...\n")

    footer("pncq",6)
end procedure

procedure page7()
    header("demo/tutorial : attributes")
    select_font(sysb)
    set_foreground(BRIGHT_WHITE)
    set_background(BLUE)
    setx(10) sety(60)
    write("There are four distinct attributes currently recognized by the system. Each has a constant asigned\n")
    write("to it for easier manipulation:\n\n")
    select_font(ROM)
    write("    NONE      = 0  -- not an attribute, just lack of it\n")
    write("    BOLD      = 1\n")
    write("    ITALIC    = 2\n")
    write("    SHADOW    = 4\n")
    write("    UNDERLINE = 8\n\n")
    select_font(sysb)
    write("The attributes are 'additive', they can be combined in every possible way. Perhaps, a couple of real\n")
    write("examples would be the best illustration:\n\n")
    select_font(ROM)
    set_foreground(YELLOW)
    write("    set_foreground(WHITE)\n")
    write("    set_background(CLEAR)\n")
    write("    set_attributes(BOLD+ITALIC+SHADOW)\n")
    write("    setx(40)\n")
    write("    write(\"This text is in bold italics with shadows...\\n\")\n")
    write("    set_attributes(SHADOW+UNDERLINE)\n")
    write("    write(\"This text has shadows and is underlined...\")\n\n")
    select_font(sysb)
    set_foreground(BRIGHT_WHITE)
    set_background(BLUE)
    write("And this is what we get:\n\n")
    set_foreground(WHITE)
    set_background(CLEAR)
    set_attributes(BOLD+ITALIC+SHADOW)
    setx(40)
    write("This text is in bold italics with shadows...\n")
    set_attributes(SHADOW+UNDERLINE)
    write("This text has shadows and is underlined...")

    footer("pncq",7)
end procedure

procedure page8()
    header("demo/tutorial : attributes")
    select_font(sysb)
    set_foreground(BRIGHT_WHITE)
    set_background(BLUE)
    setx(10) sety(60)
    write("The shadows often effectively double the time required to display the text, so in the fast, opaque\n")
    write("mode the SHADOW attribute has "&-1&"no"&0&" effect - it is not cancelled, it is simply ignored! (Note: this is the\n")
    write("only "&-1&"functional"&0&" difference between the modes. I'll fix it as soon as I find fast enough algorithm...)\n\n\n")

    write("Just like foreground colors, the attribute can also be changed 'on-the-fly': any char value equal to or\n")
    write("smaller than zero resets the combined attribute; its absolute value is the new attribute!\n\n")

    select_font(ROM)
    set_foreground(YELLOW)
    write("    set_foreground(WHITE)\n")
    write("    set_background(CLEAR)\n")
    write("    set_attributes(SHADOW)\n")
    write("    setx(40)\n")
    write("    write(\"the quick \"&-13&\"brown\"&0&\" fox jumps \"&-6&\"over a lazy dog...\")\n\n")

    set_foreground(BRIGHT_WHITE)
    set_background(BLUE)
    write("And this is the result:\n\n")
    set_foreground(WHITE)
    set_background(CLEAR)
    set_attributes(SHADOW)
    setx(40)
    write("the quick "&-13&"brown"&0&" fox jumps "&-6&"over a lazy dog...\n\n")

    set_foreground(BRIGHT_WHITE)
    set_background(BLUE)
    set_attributes(NONE)
    setx(10)
    write("The text starts, as specified, with the SHADOW attribute only.\n")
    write("The first insert is  -(BOLD+SHADOW+UNDERLINE) = -(1+4+8) = -13\n")
    write("The second reverts to plain text only, NONE = 0.\n")
    write("And the third one takes us back to shadows, but this time combined with\n")
    write("italics:  -(ITALIC+SHADOW) = -(2+4) = -6")

    footer("pncq",8)
end procedure

procedure page9()
    header("demo/tutorial : justification")
    select_font(sysb)
    set_foreground(BRIGHT_WHITE)
    set_background(BLUE)
    setx(10) sety(60)
    write("It is mainly multi-line strings, the ones with imbedded newlines, that are handled very differently at\n")
    write("different line justifications. Naturally enough there are just three of them: LEFT, RIGHT and CENTRE.\n")
    write("And they are all relative to the "&-1&"initial"&0&" location of the text pointer. First of all, some simple code:\n\n")
    select_font(ROM)
    set_foreground(YELLOW)
    write("    write(\"This is\\nthe default\\nleft justification\\n\\n\")\n")
    write("    justify(CENTRE)\n")
    write("    write(\"This text is\\ncentred\\nabout the same x-coordinate\\n\\n\")\n")
    write("    justify(RIGHT)\n")
    write("    write(\"And this is\\nright\\njustified\\n\\n\")\n\n")

    select_font(sysb)
    set_foreground(BRIGHT_WHITE)
    set_background(BLUE)
    write("And this is how it looks:\n\n")
    bar(10,245,169,190,BRIGHT_BLUE)
    bar(180,245,169,190,BRIGHT_MAGENTA)
    set_foreground(WHITE)
    set_background(CLEAR)
    set_attributes(SHADOW)
    setx(180)
    write("This is\nthe default\nleft justification\n\n")
    justify(CENTRE)
    write("This text is\ncentred\nabout the same x-coordinate\n\n")
    justify(RIGHT)
    write("And this is\nright\njustified\n\n")
    justify(LEFT)   -- back to default

    footer("pncq",9)
end procedure

procedure page10()
    header("demo/tutorial : orientation")
    select_font(sysb)
    set_foreground(BRIGHT_WHITE)
    set_background(BLUE)
    setx(10) sety(60)
    write("The default line orientation is obviously HORIZONTAL. We have seen plenty of that already, so let's\n")
    write("concentrate on the VERTICAL orientation. In it only one direction is supported,  common in technical\n")
    write("drawings: facing left and progressing from the bottom upwards!\n\n")
    write("The faster opaque mode is not available in this orientation, it simply defaults to the CLEAR mode,\n")
    write("temporarily, regardless of what background is currently specified! Generally, it is  un-optimized, so\n")
    write("a bit slower, but otherwise it has all the features of the horizontal text: plain color as well as tiled\n")
    write("foregrounds and all attributes. Even the three line justifications are available, so we can re-use the\n")
    write("code from the previous page for our next example, with just one or two minor modifications:\n\n")
    select_font(ROM)
    set_foreground(YELLOW)
    write("    set_orientation(VERTICAL)\n")
    write("    write(\"This is\\nthe default\\nleft justification\\n\\n\")\n")
    write("    justify(CENTRE)\n")
    write("    write(\"This text is\\ncentred\\nabout the same y-coordinate\\n\\n\")\n")
    write("    justify(RIGHT)\n")
    write("    write(\"And this is\\nright\\njustified\\n\\n\")\n\n")
    write("    -- reset to defaults\n")
    write("    set_orientation(HORIZONTAL)\n")
    write("    justify(LEFT)\n\n")

    select_font(sysb)
    set_foreground(BRIGHT_WHITE)
    write("I hope the code is self-explanatory!\n\n")
    write("And the result is shown on the next page...")
    footer("pncq",10)
end procedure

procedure page11()
    header("demo/tutorial : orientation")
    select_font(sysb)
    set_foreground(BRIGHT_WHITE)
    set_background(BLUE)
    setx(10) sety(70)
    write("Example from the previous page of all three justifications in vertical orientation:")

    bar(10,120,620,140,BRIGHT_BLUE)
    bar(10,261,620,140,BRIGHT_MAGENTA)

    select_font(mo28)
    set_orientation(VERTICAL)
    set_foreground(BRIGHT_RED)
    justify(CENTRE)
    setx(70) sety(260)
    write("vertical text")

    select_font(sysb)
    set_foreground(WHITE)
    set_background(CLEAR)
    set_attributes(SHADOW)
    setx(160)  sety(260)
    justify(LEFT)
    write("This is\nthe default\nleft justification\n\n\n")
    justify(CENTRE)
    write("This text is\ncentred\nabout the same y-coordinate\n\n\n")
    justify(RIGHT)
    write("And this is\nright\njustified")

    -- reset to defaults
    set_orientation(HORIZONTAL)
    justify(LEFT)

    footer("pncq",11)
end procedure

procedure page12()
    header("demo/tutorial : parameters")

    select_font(sysb)
    set_foreground(BRIGHT_WHITE)
    set_background(BLUE)
    setx(10) sety(60)
    write("The font parameters are a set (sequence) of size dependant variables, and each loaded font has its\n")
    write("own, private copy, which is initially set to default values:\n\n")

    select_font(ROM)
    set_foreground(WHITE)
    write("    DEFAULT_PARAMS = {1,  -- 1: boldface offset\n")
    write("                      1,  -- 2: shadow offset: right\n")
    write("                      1,  -- 3: shadow offset: down\n")
    write("                      0,  -- 4: extra kerning space width\n")
    write("                      1,  -- 5: underline thickness\n")
    write("                      0}  -- 6: extra line spacing\n\n")

    select_font(sysb)
    set_foreground(BRIGHT_WHITE)
    write("Current font parameters can be changed either individually, or the whole bunch of them at the time,\n")
    write("using the "&-1&"set_parameter()"&0&" procedure.\n")
    write("And the lot can be retrieved, if desired, by the "&-1&"parameters()"&0&" function.\n\n")
    write("Please, compare the example below with the page title, both using the same font:\n\n")

    select_font(ROM)
    set_foreground(YELLOW)
    write("    select_font(mo28)\n")
    write("    set_parameter({2,3,4},{5,5,3})\n")
    write("    set_attributes(SHADOW)\n")
    write("    set_foreground(GREEN)   set_background(CLEAR)\n")
    write("    write(\"larger shadow offset & extra kerning space\\n\")\n\n")

    select_font(mo28)
    set_parameter({2,3,4},{5,5,3})  -- right, up, extra kerning space
    set_foreground(GREEN)
    set_background(CLEAR)
    set_attributes(SHADOW)
    write("larger shadow offset & extra kerning space\n")
    set_parameter({2,3,4},{2,2,0})   -- restore font parameters

    footer("pncq",12)
end procedure

procedure page13()
    header("demo/tutorial : prompt")
    select_font(sysb)
    set_foreground(BRIGHT_WHITE)
    set_background(BLUE)
    setx(10) sety(60)
    write("The "&-1&"prompt()"&0&" is a single line text input device, the old "&-1&"fgets()"&0&" function, only fancier. And sorry, but \n")
    write("I have just changed it again, for the third time! It takes two arguments, the "&-1&"initial prompt"&0&" and the \n")
    write("maximum field "&-1&"width"&0&". Also, at Irv's request, it now returns a two element sequence. The first element\n")
    write("is the prompt terminator, which can be any of the following six keys:  "&-1&"enter, esc, tab, shift-tab,\n")
    write(0&"and "&-1&"up"&0&" or "&-1&"down"&0&" arrow. The second element is the edited prompt string itself.\n\n")

    write("The prompt effectively "&-1&"ignores"&0&" all current attributes, but they are safely "&-1&"restored"&0&" on exit. For\n")
    write("editing the system must have the "&-1&"paper"&0&" color. It is taken to be the "&-1&"background"&0&" color, and if that\n")
    write("happens to be "&-1&"CLEAR"&0&", the actual screen background is sampled at the initial text pointer location!\n\n")

    write("The field width does not really limit the total length of the prompt, because the string is fully\n")
    write("scrollable. All the usual editing keys behave as expected, so let's try another example:\n\n")

    select_font(ROM)
    set_foreground(YELLOW)
    write("    bar(10,365,210,25,BRIGHT_WHITE)\n")
    write("    setx(15) sety(370)\n")
    write("    set_foreground(RED)\n")
    write("    set_background(BRIGHT_WHITE)\n")
    write("    junk=prompt(\"My name is \",200)\n\n")

    select_font(sysb)
    bar(10,365,210,25,BRIGHT_WHITE)
    setx(15) sety(370)
    set_foreground(RED)
    set_background(BRIGHT_WHITE)
    junk=prompt("My name is ",200)

    set_foreground(WHITE)
    set_background(BLUE)
    setx(10) sety(405)
    write("Escape key: "& #10F & sprintf("%d",junk[1]))
    set_foreground(WHITE)
    setx(10) sety(425)
    write("String: "& #10F & junk[2])

    footer("pncq",13)
end procedure

procedure page14()
    header("demo/tutorial : odds and ends")
    select_font(sysb)
    set_foreground(BRIGHT_WHITE)
    set_background(BLUE)
    setx(10) sety(60)
    write("As indicated in the title, here we will deal with a collection of elements not adequately covered in the\n")
    write("preceding pages. Let's take them in the alphabetical order, more or less:\n\n")

    write(-1&"align()"&0&" - A convenience routine that takes a single argument: a valid "&-1&"font handle"&0&".  It uses "&-1&"baseline\n")
    write("heights"&0&" to align the "&-1&"current"&0&" font with the one indicated by the "&-1&"handle"&0&". Look:\n\n")
    write("This is not aligned:  ")
    select_font(mo28)
    set_foreground(GREEN)
    write("euphoria")
    select_font(sysb)
    set_foreground(BRIGHT_WHITE)
    write("    But this is:  ")
    select_font(mo28)
    align(sysb)
    set_foreground(GREEN)
    write("euphoria\n\n")

    select_font(sysb)
    setx(10)
    set_foreground(BRIGHT_WHITE)
    write(-1&"baseline_height(), height(), width()"&0&" - Three potentially useful functions, I am not really quite sure\n")
    write("what to say...\n\n")

    write(-1&"images()"&0&" - Something that David Cuny insisted on, for very obscure reasons...;-) It is a function that\n")
    write("takes a (multi-line) string as its only argument, and returns a sequence of 2-d images corresponding to\n")
    write("the (multi-line) string. Presumably, these images can be massaged further, before they are displayed.\n")
    write("The method of their generation is the same as for the text in the "&-1&"opaque"&0&" mode, in fact it can be\n")
    write("viewed as a sort of fast mode with a 'delayed' or 'postponed' display...\n\n")

    write(-1&"lengths()"&0&" - A very important function that returns a sequence of pixel lengths of all "&-1&"substrings"&0&" of\n")
    write("the input string. The number of substrings is the same as the number of imbedded "&-1&"newlines"&0&" that\n")
    write("divide the input string "&-1&"plus 1 "&0&"!\n\n")

    write(-1&"reset_font()"&0&" - Just what it says: the procedure resets the entire environment of the current font\n")
    write("to its (initial) defaults.\n")

    footer("pncq",14)
end procedure

procedure page15()
    junk=graphics_mode(19)
    cls(BLACK)
    set_fire_palette()
    -- write in black on black, invisibly
    all_palette(allblack)
    select_font(mo28)
    set_foreground(tile-4)
    set_background(CLEAR)
    setx(160) sety(85)
    justify(CENTRE)
    write("euphoria")
    justify(LEFT)                   -- restore default justification
    -- fade-in over 50 steps
    for i=1 to 50 do
        t=time()+0.06               -- 3 second fade-in
        all_palette(floor(0.02*i*pal))
        while time()<t do end while -- waste time
    end for
    fire(109,85,102,28)
end procedure


-- initialize
tick_rate(100)                      -- better time resolution
sysb=load_font("sysb13.f")
mo28=load_font("modern28.f")
sc24=load_font("scotia24.f")
junk=read_bitmap("circles.bmp")
circles_pal=floor(junk[1]/4)
circles=junk[2]
allblack=repeat({0,0,0},256)        -- totaly black palette

-- construct a nice foreground tile...
tile={}
for i=1 to 28 do
    tile=append(tile,{i+55})
end for

select_font(mo28)
set_parameter({1,2,3},{2,2,2})
select_font(sysb)
set_parameter(6,3)                  -- extra line spacing

-- main loop
junk=graphics_mode(18)
page=0
while TRUE do
    if    page=0  then burn()
    elsif page=1  then contents()
    elsif page=2  then page2()      -- itroduction
    elsif page=3  then page3()      -- basics
    elsif page=4  then page4()
    elsif page=5  then page5()
    elsif page=6  then page6()      -- modes
    elsif page=7  then page7()      -- attributes
    elsif page=8  then page8()
    elsif page=9  then page9()      -- justification
    elsif page=10 then page10()     -- orientation
    elsif page=11 then page11()
    elsif page=12 then page12()     -- parameters
    elsif page=13 then page13()     -- prompt
    elsif page=14 then page14()     -- odds & ends
    elsif page=15 then              -- the last word
        page15() exit
    else  exit
    end if
end while
junk=graphics_mode(-1)
